/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 * C Example Program : CDMA2000 Non Signalling Measurements
 *
 * The following example explains how to handle primary and secondary
 * addressing and how to perform a simple measurement with the CMU. In the
 * example, remote control via GPIB bus and the programming language C is used.
 *
 * The example requires CMU options :
 *	
 *	CMU200
 *  CMU-B83	- CDMA2000 LH
 *	CMU-K84 - SW package for CDMA2000 US cellular band
 *  CMU-K85 - SW package for CDMA2000 PCS 1900 band
 *  
 *	FW 3.05 or greater
 *
 *  rscmu200 CVI driver - Rev 3.10.06
 *  rscmuc2k CVI driver - Rev 3.11.01 
 *
 * Compiler name and release(s) : LabWindows/CVI 5.5
 * Original Release : January, 2002
 *
 * By ROHDE & SCHWARZ
 *      CONTACTS :
 *              Support & Application
 *              Rohde & Schwarz France
 *              Parc Tertiaire de Meudon
 *              25-27, rue Jeanne Braconnier
 *              92366 Meudon-la-Foret
 *              Telephone : 00 33 1 41 36 10 29
 *              Fax       : 00 33 1 41 36 11 10
 *              e-Mail    : GSA@RSF.RSD.DE
 *
 * Version : 1.1  03/2003
 *
 * Modifikation:
 * changed close function from RSCMU_CDMA_Close to RSCMU_CDMA2K_Close
 *   by Werner Klotzsche  Rohde & Schwarz
 *				  
 * (C)opyright 2002 ROHDE & SCHWARZ FRANCE. All Rights Reserved.
 *
 * ----------------------------------------------------------------------------*/
#include "ansi_c.h"
#include "userint.h"
#include "utility.h"
#include "rscmu200.h"
#include "rscmuc2k.h"

// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif


// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
	ViChar l_vCmuErrMsg[1024];
	ViChar l_vErrBuf[1024];
	ViInt32 _error;
    
	if (CmuStatus != VI_SUCCESS)
	{

		//RSCMU_Base_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
		RSCMU_CDMA_errorMessage(VI_NULL, CmuStatus, l_vCmuErrMsg);

		//Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
		sprintf (l_vErrBuf, "Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);        
		MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);
	
	return -1;
	}
    return 0;
    
Error:
    return -1;

}



// Structure handle
typedef struct {

	ViSession	BASE;
	ViSession	RF;
	ViSession	AUDIO;
	ViSession	GSM400S;
	ViSession	GSM400NS;
	ViSession	GSM850S;
	ViSession	GSM850NS;
	ViSession	GSM900S;
	ViSession	GSM900NS;
	ViSession	GSM1800S;
	ViSession	GSM1800NS;
	ViSession	GSM1900S;
	ViSession	GSM1900NS;
	ViSession	CDMA800S;
	ViSession	CDMA800NS;	
	ViSession	CDMA1900S;
	ViSession	CDMA1900NS;
	ViSession	VALID;

} tHandle;



typedef struct {

	ViReal64 AvgCur;
	ViReal64 MinCur; 
	ViReal64 MaxCur;
	ViReal64 AvgAvg; 
	ViReal64 MinAvg; 
	ViReal64 MaxAvg; 

} tNPOW;



int main(void) 
{
	
	// Init variable
	tHandle				handle 			= {NULL};
	tNPOW				NPOW 			= {NULL};
	ModQualOver 		TX_MOD_QUAL		= {NULL};
	
	ViChar				e[1024]			= "\0";
	ViChar				buffout[512]	= "\0";
	ViChar				buffin[512]		= "\0";
	ViChar				OPT[1024]		= "\0"; 
	
	ViStatus			cmu_status		= 0;
	
	int					_error;	
	int					TestLoop		= 0;
	int					DispStatus		= 0;
	ViReal64			CMURFLevel 		= 0.0;

	/************************************************************************************************/
	//
	/************************************************************************************************/
	
	Cls();
	printf("\nR&S CMU200 - CDMA2000 Non Signalling example\n");
	
	
	// Init the base driver (open handle with the rscmu200.dll)
	printf("\tGet handle with the base driver rscmu200.dll\n");
	CheckCmuError(RSCMU_Init_Base (&handle.BASE, RSCMU_GPIB, 0, 20,RSCMU_DO_QUERY, RSCMU_RESET_OFF, e));

	// Open trace file and GPIB exchange
	printf("\tStore the GPIB exchange in a log file\n");
	CheckCmuError(RSCMU_Trace_ON (handle.BASE, "RsCmu_C2kNSIG.iee", e));
	 
	// Retreive CMU options
	printf("\tRetreive the CMU options\n");
	CheckCmuError(RSCMU_Query (handle.BASE, "*OPT?", OPT, RSCMU_OFF, e));
	 
	// Test if CMU supports CD2K
	if (strstr(OPT,"B83") == NULL ) 
	{
		strcpy(e,"Option B83 required for this example\0");
		goto Error;
	};
	
	// Test if CMU supports K84 SW package
	if (strstr(OPT,"K84") == NULL ) 
	{
		strcpy(e,"Example required K84 SW package CDMA2K US cell band\0");
		goto Error;
	};

	// Test if CMU supports K85 SW package
	if (strstr(OPT,"K85") == NULL ) 
	{
		strcpy(e,"Example required K85 SW package CDMA2K PCS band\0");
		goto Error;
	};
	
	/************************************************************************************************/
	// Map the GPIB secondary addresses with the CDMA200 K84 and K85 functionnal group
	/************************************************************************************************/

	// Map the secondary addresses
	// Map the US cellular Non Signalling Group
	printf("\tAssign secondary adresses\n");
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 2, RSCMU_CDMA2KCELLMS_NSIG, e))
	// Map the US cellular Signalling Group
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 3, RSCMU_CDMA2KCELLMS_SIG, e));
	// Map the PCS cellular Non Signalling Group
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 4, RSCMU_CDMA2KPCSMS_NSIG, e));
	// Map the PCS cellular Non Signalling Group
	CheckCmuError(RSCMU_Base_SystRemAddrSec (handle.BASE, 5, RSCMU_CDMA2KPCSMS_SIG, e));

	/************************************************************************************************/
	// Get handles
	/************************************************************************************************/
	printf("\tGet handle for CDMA800 Nsig group & CDMA1900 NSig group (open handles with the rscmuc2k.dll)\n");
	CheckCmuError(RSCMU_Init_CDMA2K (handle.BASE, &handle.CDMA800NS, RSCMU_CDMA2KCELLMS_NSIG, e));
	CheckCmuError(RSCMU_Init_CDMA2K (handle.BASE, &handle.CDMA1900NS,RSCMU_CDMA2KPCSMS_NSIG, e));
	
	// Set Timeout
	printf("\tSet Timeout\n");
	CheckCmuError(RSCMU_SetTimeoutAttribute (handle.CDMA800NS, 7000, e));
	CheckCmuError(RSCMU_SetTimeoutAttribute (handle.CDMA1900NS, 7000, e));
	
	
	/************************************************************************************************/
	// Set Network to US cellular
	/************************************************************************************************/
	handle.VALID = handle.CDMA800NS;   
	CheckCmuError(RSCMU_CDMA2K_ConfNstandard (handle.VALID, RSCMU_C2K_USC, e));
	

	/************************************************************************************************/
	// Set RF Path
	/************************************************************************************************/
	// Select the RF input and Output and Set the RF cable compensation
	printf("\nSet RF path and RF attenuation\n");
	CheckCmuError(RSCMU_CDMA2K_InpState (handle.VALID, RSCMU_INPUT_RF2, e));
	CheckCmuError(RSCMU_CDMA2K_SenseCorrLossInp (handle.VALID, RSCMU_INPUT_RF2, 1.0, e));
	
	CheckCmuError(RSCMU_CDMA2K_OutpState (handle.VALID, RSCMU_OUTPUT_RF3, e));
	CheckCmuError(RSCMU_CDMA2K_SourceCorrLossOutp (handle.VALID, RSCMU_OUTPUT_RF3, 0.0,e));

	/************************************************************************************************/
	// Configure the RF generator
	/************************************************************************************************/
	printf("\nConfigure the RF Generator\n");
	
	// Abort RFG
	printf("\tAbort RFG\n");
	CheckCmuError(RSCMU_CDMA2K_AbortRfGenerator (handle.VALID, e));
	
	// Set RF Channel
	printf("\tSet RF Channel\n");
	CheckCmuError(RSCMU_CDMA2K_SourRfGenRfChannel (handle.VALID, 384, e));
	
	// Set RF gen level
	printf("\tSet RF genertator Level\n");
	CheckCmuError(RSCMU_CDMA2K_SourRfGenRfPowCDMA (handle.VALID, -16.0, e));
	
	// Stop OCNS
	CheckCmuError(RSCMU_CDMA2K_SourRfGenRfLevOCNSEnable (handle.VALID,RSCMU_MODE_ON, e));
	
	/************************************************************************************************/
	// Configure the RF analyzer
	/************************************************************************************************/
	printf("\nConfigure the RF OQPSK analyzer\n");

	
	// Set Mode AUTO ranging OFF
	printf("\tMode AUTO ranging OFF\n");
	CheckCmuError(RSCMU_CDMA2K_SenseLevMode (handle.VALID, RSCMU_MANUAL, e));
	
	// Set the power level range   = RMS power expected + 10 dB marge (crest factor)
	printf("\tSet the power level range\n");
	CheckCmuError(RSCMU_CDMA2K_SenseLevMax (handle.VALID, 0.0, e));
	
	// Set trigger to FREE RUN 
	printf("\tSet trigger to FREE RUN\n");
	CheckCmuError(RSCMU_CDMA2K_TrigSeqSourOQPSK (handle.VALID, RSCMU_TRIG_SOUR_FRUN,e));

	// Set RF frequency
	printf("\tSet RF frequency\n");	
	CheckCmuError(RSCMU_CDMA2K_SenseRfanFreq (handle.VALID, 881.52, e));

	/************************************************************************************************/
	// Configure Measurement
	/************************************************************************************************/
	printf("\nConfigure Measurement(s)\n");	
	printf("\tConfigure OQPSK Analyzer\n");	
	CheckCmuError(RSCMU_CDMA2K_ConfModMQualCont (handle.VALID, 
												10, 
												RSCMU_REP_SINGLESHOT,
							   					RSCMU_STOP3_SON, 
							   					RSCMU_STEP_MODE_NONE,
							   					RSCMU_OQPSK, e));
							   					
	
	// Setup Power meter
	printf("\tConfigure NPOWER\n");	
	CheckCmuError(RSCMU_CDMA2K_ConfNPowCont (handle.VALID, 
												5, 
												RSCMU_REP_SINGLESHOT,
												RSCMU_NONE, 
												RSCMU_NONE, 
												e));
	
	
	
	// Start RFG
		printf("\nStart RF generator\n");
	CheckCmuError(RSCMU_CDMA2K_InitRfGenerator (handle.VALID, e));

	
	

TestLoop = 1;
do
{

	// Display Message and Query item
	DispStatus = GenericMessagePopup ("CDMA2K - NSIG Example","\nConnect ports RF3 OUT and RF2 with a N cable\n\nFor a new test Press <OK>\nFor exit <Cancel>","OK", "Cancel", "", 
			0, 0, 0,
			VAL_GENERIC_POPUP_BTN1,
			VAL_GENERIC_POPUP_BTN1,
			VAL_GENERIC_POPUP_BTN2);
			
	if (DispStatus == 2) {
		TestLoop = 0;
		break;
	};
	
	printf("\nTX Test Modulation and POWER using Modulation Quality OQPSK analyzer  and NPOWER Objects\n");
	
	// Loop decread RF generator level and measure the forward power using the NPOWer object
	CMURFLevel = 0.0;
	do
	{
		
		// set the power meter range  
		// RMS level = CMURFLevel + 10 dB.
		CheckCmuError(RSCMU_CDMA2K_SenseLevMax (handle.CDMA800NS, CMURFLevel + 10.0, e));
		
		// Set RF level
		CheckCmuError(RSCMU_CDMA2K_SourRfGenRfPowCDMA (handle.CDMA800NS, CMURFLevel, e));	
		
		if (CMURFLevel == 0.0)
		{
			
			// Trig a Modulation measurement
			printf("\tTrig a Modulation measurement\n");
			CheckCmuError(RSCMU_CDMA2K_ReadModMQuality (handle.VALID, 
												&TX_MOD_QUAL, 
												RSCMU_OQPSK,
							  					e));
							  					
							  					
printf("\
\tPower\t\t\t : %6.2lf dBm\n\
\tWaveform Qual\t : %6.3lf\n\
\tCarrier FeedThr\t : %6.2lf dB\n\
\tLow Side Band \t : %6.2lf dB\n\
\tLow Side Band \t : %6.2lf dB\n\n",
TX_MOD_QUAL.msPower.Average,
TX_MOD_QUAL.WaveFormQuality.Average,
TX_MOD_QUAL.CarrierFeedThrough.Average,
TX_MOD_QUAL.SideBandSuppresLower.Average,
TX_MOD_QUAL.SideBandSuppresUpper.Average
);

			// Abort DSP
			CheckCmuError(RSCMU_CDMA2K_AbortModMQuality (handle.VALID, RSCMU_OQPSK, e));
			
		}	
		// Trig a power measurement
		CheckCmuError(RSCMU_CDMA2K_ReadScalNPow (handle.CDMA800NS, 
													&NPOW.AvgCur, 
													&NPOW.MinCur, 
													&NPOW.MaxCur, 
													&NPOW.AvgAvg, 
													&NPOW.MinAvg, 
													&NPOW.MaxAvg, 
													e));
	
	
		printf("\tCMU RF Level %6.2lf dBm - Power : %6.2lf dBm \n", CMURFLevel, NPOW.AvgAvg);
	
		CMURFLevel = CMURFLevel - 2.0;
		
	} while (CMURFLevel > -60.0);
	
	// ABORT NPOWER
	CheckCmuError(RSCMU_CDMA2K_AbortNPow (handle.VALID, e));
	
	

} while (TestLoop == 1);

	// Stop RFG
	CheckCmuError(RSCMU_CDMA2K_AbortRfGenerator (handle.CDMA800NS, e));	

	// Close trace
	if (handle.BASE !=0) RSCMU_Trace_OFF (handle.BASE, e);	
	
							  
	if (handle.CDMA800NS != 0) RSCMU_CDMA2K_Close (handle.CDMA800NS, e);
	if (handle.CDMA800S != 0) RSCMU_CDMA2K_Close (handle.CDMA800S, e);
	if (handle.BASE !=0) RSCMU_Close_Base (handle.BASE, e);
	
	
	return 0;

Error:	

	MessagePopup ("Error", e);
	
	// Close trace
	if (handle.BASE !=0) RSCMU_Trace_OFF (handle.BASE, e);	
	
	if (handle.CDMA800NS != 0) RSCMU_CDMA2K_Close (handle.CDMA800NS, e);
	if (handle.CDMA800S != 0) RSCMU_CDMA2K_Close (handle.CDMA800S, e);
	if (handle.BASE !=0) RSCMU_Close_Base (handle.BASE, e);
	
	
	return -1;
}
